==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2310_174.zip: ronportal/ronportal/dovednosti/dovednostibrowser.php <==
<?php
/**
 * pavel.rudol, 31.10.2022 16:01
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DB\Cursor;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\MVC\DataOutputObject;
use Ron\MVC\OutputObject;
use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\Prava;
use Ron\TextManager;

class SmazaniDovednostiField extends BrowserField
{
    private $osobaId;
    private $osobaRC;

    /**
     * SmazaniDovednostiField constructor.
     * @param null|Database $database
     * @param int $osobaId
     * @param string $osobaRC
     */
    public function __construct($database, $osobaId, $osobaRC)
    {
        parent::__construct($database);

        $this->osobaId = $osobaId;
        $this->osobaRC = $osobaRC;
    }

    /** Zobrazi odkaz pro smazani ziskane dovednosti. Smazani je mozne jen pokud neni pripojen dokument.
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!DovednostiController::jePripojenDokumentDovednosti($this->database, $this->osobaRC, $rowData['col-dovednost-kod'], $rowData['col-uroven-kod'],
            $this->database->stringToTimestamp($rowData['col-datumziskani']))) {

            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_zaznam') . '"></i>',
                sprintf(TextManager::_('Dovednosti_smazat'), $rowData['col-dovednost-popis']),
                '?menuid=maticedovednosti&action=zrusitdovednost&osoba=' . $this->osobaId . '&dovednost=' . HttpUtils::urlEncode($rowData['col-dovednost-kod']) .
                '&uroven=' . $rowData['col-uroven-kod'] . '&datumziskani=' . $this->database->stringToTimestamp($rowData['col-datumziskani'])
            );
        }

        return '';
    }
}

/**
 * Seznam platnych dovednosti osoby
 */
class DovednostiBrowser extends Browser implements OutputObject
{
    private $pravaEditace;
    private $pravaZobrazeni;
    private $pravaSmazani;

    private $idOsoby;

    /**
     * @param Database $database
     * @param string $url
     * @param int $idOsoby id PP osoby
     * @throws DBException
     * @throws Exception
     */
    function __construct($database, $url, $idOsoby)
    {
        parent::__construct($database);

        $this->pravaEditace = Prava::vratPrava('doved_edit');
        $this->pravaZobrazeni = Prava::vratPrava('doved_ziskane');
        $this->pravaSmazani = Prava::vratPrava('doved_smazani');

        Prava::nastavObdobiProPrava(time(), time());

        $this->showMessages = false;

        $this->idOsoby = $idOsoby;

        $this->title = TextManager::_('Dovednosti');

        $this->browserUrl = $url;
        $this->code = 'browserdovednostiosoby';

        $this->sqlFromPart =
            "DovednostOsoby, UrovenDovednosti, Dovednost ";

        $this->sqlWherePart =
            " (UrovenDovednosti.KodDovednosti = DovednostOsoby.KodDovednosti) " .
            " AND (UrovenDovednosti.KodUrovne = DovednostOsoby.KodUrovne) " .
            " AND (UrovenDovednosti.KodDovednosti = Dovednost.Kod) ";

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('DovednostOsoby.KodDovednosti')->setCode('col-dovednost-kod');
        $this->addField()
            ->setSqlFieldName('Dovednost.Popis')->setHeaderText('Dovednost')->setAlign('left')
            ->setOrdering(true)->setCode('col-dovednost-popis');
        $this->addField()
            ->setSqlFieldName('UrovenDovednosti.Popis')->setHeaderText('Dovednosti_uroven_dovednosti')->setAlign('left')
            ->setOrdering(true)->setCode('col-uroven-popis');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.KodUrovne')->setHeaderText('Kod')->setAlign('right')
            ->setOrdering(true)->setCode('col-uroven-kod');
        $this->addField()
            ->setSqlFieldName('(SELECT Popis FROM S_SpecUrovneDovednosti WHERE Kod = DovednostOsoby.SpecifikaceUrovne) as SpecifikaceUrovne')
            ->setSortSQL('SpecifikaceUrovne')
            ->setHeaderText('Dovednosti_specifikace_urovne')->setAlign('right')
            ->setOrdering(true)->setCode('col-uroven-specifikace');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.DatumZiskani')->setHeaderText('Date')->setAlign('right')
            ->setOrdering(true)->setCode('col-datumziskani')->setFormat('finedate');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.PlatnostDo')->setHeaderText('Platnost_do')->setAlign('right')
            ->setOrdering(true)->setCode('col-platnostdo')->setFormat('finedate');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Insert_UserName')->setHeaderText('Dovednosti_vlozil_uzivatel')->setAlign('right')
            ->setOrdering(true)->setCode('col-insert-username');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Insert_Timestamp')->setHeaderText('Date')->setAlign('right')
            ->setOrdering(true)->setCode('col-insert-timestamp')->setFormat('finedatetime');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Update_UserName')->setHeaderText('Dovednosti_aktualizoval_uzivatel')->setAlign('right')
            ->setOrdering(true)->setCode('col-update-username');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Update_UserName')->setHeaderText('Date')->setAlign('right')
            ->setOrdering(true)->setCode('col-update-timestamp')->setFormat('finedatetime');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(DovednostOsoby.Poznamka)}')
            ->setCode('col-poznamka')
            ->setHeader(TextManager::_('Poznamka'))
            ->setClassName('poznamka do_poznamka')
            ->setHeaderColSpan(1);
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(UrovenDovednosti.RozsirenyPopis)}')
            ->setCode('col-rozsirenypopis')
            ->setClassName('poznamka ud_poznamka')
            ->setHeader(TextManager::_('Dovednosti_rozsireny_popis_urovne'))
            ->setHeaderClass('font-weight-bold font-italic')
            ->setHeaderColSpan(1);

        if ($this->pravaSmazani->jsouNejakePrava()) {
            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($this->idOsoby), $params);
            if ($this->pravaSmazani->lzeZobrazitRC($osobaRC)) {
                $this->addField(new SmazaniDovednostiField($this->database, $this->idOsoby, $osobaRC));
            }
        }

        $this->defaultSort = '-col-datumziskani';
        $this->primaryField = 0;
    }


    /** Doplni filtr podle prav na zobrazeni, editaci.
     * @return SqlStatement
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        return new SqlStatement(
            " EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer WHERE PracovniPomer.RC = DovednostOsoby.RC AND PracovniPomer.Id = " . $params->addParameter($this->idOsoby) .
            " AND ((" . $this->pravaEditace->sqlFiltr('PracovniPomer', time(), time(), $params) . ") OR " .
            "      (" . $this->pravaZobrazeni->sqlFiltr('PracovniPomer', time(), time(), $params) . "))" .
            ") ", $params);
    }


    /** Doplni do hlavicky odkaz ke stazeni dat v CSV.
     * @return void
     */
    function renderHeader()
    {
        parent::renderHeader();

        echo "<a href='?menuid=dovednosti&action=downloadcsv&osoba=$this->idOsoby'>" . TextManager::_('Dovednosti_csv') . "</a>";
    }

    /** Sestavi nadpisy sloupcu v CSV souboru.
     * @return string
     */
    public function outputHeader()
    {
        $poms = '';

        foreach ($this->fields as $field) {
            if ($poms) {
                $poms .= ';';
            }

            $poms .= $field->header;
        }

        return $poms . "\n";
    }

    /** Vypise radek s daty do retezce pro CSV soubor.
     * @param $values
     * @return string
     */
    public function outputLine($values)
    {
        $poms = '';

        foreach ($this->fields as $field) {
            $value = null;

            if ($field->sqlFieldIndex >= 0 && isset($values[$field->sqlFieldIndex])) {
                $value = $values[$field->sqlFieldIndex];
            }

            if ($poms) {
                $poms .= ';';
            }

            $poms .= ($field->format ? DBUtils::getFormatedValue($this->database, $field->format, $value) : $value);
        }

        return $poms;
    }

    /** Generuje radky z SQL dotazu do retezce pro CSV soubor.
     * @param Cursor $cursor
     * @return string
     * @throws Exception
     */
    public function outputCursor($cursor)
    {
        $data = '';

        if (!$cursor) {
            throw new Exception('Chyba v metode browser::outputCursor - nejsou nactena data!');
        }

        while ($row = $cursor->fetch()) {
            if ($this->enableLine($row)) {
                $data .= $this->outputLine($row) . "\n";
            }
        }

        return $data;
    }

    /** Posle na vystup obsah browseru ve forme CSV souboru.
     * @throws Exception
     */
    public function output()
    {
        $data = $this->outputHeader() . $this->outputCursor($this->cursor);

        $csv = new DataOutputObject($data, 'dovednosti.csv', TextManager::getInstance()->getCharSet());

        $csv->output();
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2311_175.zip: ronportal/ronportal/dovednosti/dovednostibrowser.php <==
<?php
/**
 * pavel.rudol, 31.10.2022 16:01
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DB\Cursor;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\MVC\DataOutputObject;
use Ron\MVC\OutputObject;
use Ron\DB\Database;
use Ron\StringUtils;
use RonPortal\Prava;
use Ron\TextManager;

class SmazaniDovednostiField extends BrowserField
{
    private $osobaId;
    private $osobaRC;

    /**
     * SmazaniDovednostiField constructor.
     * @param null|Database $database
     * @param int $osobaId
     * @param string $osobaRC
     */
    public function __construct($database, $osobaId, $osobaRC)
    {
        parent::__construct($database);

        $this->osobaId = $osobaId;
        $this->osobaRC = $osobaRC;
    }

    /** Zobrazi odkaz pro smazani ziskane dovednosti. Smazani je mozne jen pokud neni pripojen dokument.
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!DovednostiController::jePripojenDokumentDovednosti($this->database, $this->osobaRC, $rowData['col-dovednost-kod'], $rowData['col-uroven-kod'],
            $this->database->stringToTimestamp($rowData['col-datumziskani']))) {

            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . TextManager::_('Zrusit_zaznam') . '"></i>',
                sprintf(TextManager::_('Dovednosti_smazat'), $rowData['col-dovednost-popis']),
                '?menuid=maticedovednosti&action=zrusitdovednost&osoba=' . $this->osobaId . '&dovednost=' . HttpUtils::urlEncode($rowData['col-dovednost-kod']) .
                '&uroven=' . $rowData['col-uroven-kod'] . '&datumziskani=' . $this->database->stringToTimestamp($rowData['col-datumziskani'])
            );
        }

        return '';
    }
}

/**
 * Seznam platnych dovednosti osoby
 */
class DovednostiBrowser extends Browser implements OutputObject
{
    private $pravaEditace;
    private $pravaZobrazeni;
    private $pravaSmazani;

    private $idOsoby;

    /**
     * @param Database $database
     * @param string $url
     * @param int $idOsoby id PP osoby
     * @throws DBException
     * @throws Exception
     */
    function __construct($database, $url, $idOsoby)
    {
        parent::__construct($database);

        $this->pravaEditace = Prava::vratPrava('doved_edit');
        $this->pravaZobrazeni = Prava::vratPrava('doved_ziskane');
        $this->pravaSmazani = Prava::vratPrava('doved_smazani');

        Prava::nastavObdobiProPrava(time(), time());

        $this->showMessages = false;

        $this->idOsoby = $idOsoby;

        $this->title = TextManager::_('Dovednosti');

        $this->browserUrl = $url;
        $this->code = 'browserdovednostiosoby';

        $this->sqlFromPart =
            "DovednostOsoby, UrovenDovednosti, Dovednost ";

        $this->sqlWherePart =
            " (UrovenDovednosti.KodDovednosti = DovednostOsoby.KodDovednosti) " .
            " AND (UrovenDovednosti.KodUrovne = DovednostOsoby.KodUrovne) " .
            " AND (UrovenDovednosti.KodDovednosti = Dovednost.Kod) ";

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('DovednostOsoby.KodDovednosti')->setCode('col-dovednost-kod');
        $this->addField()
            ->setSqlFieldName('Dovednost.Popis')->setHeaderText('Dovednost')->setAlign('left')
            ->setOrdering(true)->setCode('col-dovednost-popis');
        $this->addField()
            ->setSqlFieldName('UrovenDovednosti.Popis')->setHeaderText('Dovednosti_uroven_dovednosti')->setAlign('left')
            ->setOrdering(true)->setCode('col-uroven-popis');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.KodUrovne')->setHeaderText('Kod')->setAlign('right')
            ->setOrdering(true)->setCode('col-uroven-kod');
        $this->addField()
            ->setSqlFieldName('(SELECT Popis FROM S_SpecUrovneDovednosti WHERE Kod = DovednostOsoby.SpecifikaceUrovne) as SpecifikaceUrovne')
            ->setSortSQL('SpecifikaceUrovne')
            ->setHeaderText('Dovednosti_specifikace_urovne')->setAlign('right')
            ->setOrdering(true)->setCode('col-uroven-specifikace');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.DatumZiskani')->setHeaderText('Date')->setAlign('right')
            ->setOrdering(true)->setCode('col-datumziskani')->setFormat('finedate');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.PlatnostDo')->setHeaderText('Platnost_do')->setAlign('right')
            ->setOrdering(true)->setCode('col-platnostdo')->setFormat('finedate');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Insert_UserName')->setHeaderText('Dovednosti_vlozil_uzivatel')->setAlign('right')
            ->setOrdering(true)->setCode('col-insert-username');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Insert_Timestamp')->setHeaderText('Date')->setAlign('right')
            ->setOrdering(true)->setCode('col-insert-timestamp')->setFormat('finedatetime');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Update_UserName')->setHeaderText('Dovednosti_aktualizoval_uzivatel')->setAlign('right')
            ->setOrdering(true)->setCode('col-update-username');
        $this->addField()
            ->setSqlFieldName('DovednostOsoby.Update_UserName')->setHeaderText('Date')->setAlign('right')
            ->setOrdering(true)->setCode('col-update-timestamp')->setFormat('finedatetime');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(DovednostOsoby.Poznamka)}')
            ->setCode('col-poznamka')
            ->setHeader(TextManager::_('Poznamka'))
            ->setClassName('poznamka do_poznamka')
            ->setHeaderColSpan(1);
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(UrovenDovednosti.RozsirenyPopis)}')
            ->setCode('col-rozsirenypopis')
            ->setClassName('poznamka ud_poznamka')
            ->setHeader(TextManager::_('Dovednosti_rozsireny_popis_urovne'))
            ->setHeaderClass('font-weight-bold font-italic')
            ->setHeaderColSpan(1);

        if ($this->pravaSmazani->jsouNejakePrava()) {
            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($this->idOsoby), $params);
            if ($this->pravaSmazani->lzeZobrazitRC($osobaRC)) {
                $this->addField(new SmazaniDovednostiField($this->database, $this->idOsoby, $osobaRC));
            }
        }

        $this->defaultSort = '-col-datumziskani';
        $this->primaryField = 0;
    }


    /** Doplni filtr podle prav na zobrazeni, editaci.
     * @return SqlStatement
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        return new SqlStatement(
            " EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer WHERE PracovniPomer.RC = DovednostOsoby.RC AND PracovniPomer.Id = " . $params->addParameter($this->idOsoby) .
            " AND ((" . $this->pravaEditace->sqlFiltr('PracovniPomer', time(), time(), $params) . ") OR " .
            "      (" . $this->pravaZobrazeni->sqlFiltr('PracovniPomer', time(), time(), $params) . "))" .
            ") ", $params);
    }


    /** Doplni do hlavicky odkaz ke stazeni dat v CSV.
     * @return void
     */
    function renderHeader()
    {
        parent::renderHeader();

        echo "<a href='?menuid=dovednosti&action=downloadcsv&osoba=$this->idOsoby'>" . TextManager::_('Dovednosti_csv') . "</a>";
    }

    /** Sestavi nadpisy sloupcu v CSV souboru.
     * @return string
     */
    public function outputHeader()
    {
        $poms = '';

        foreach ($this->fields as $field) {
            $poms .=
                ($poms ? ';' : '') .
                $field->header;
        }

        return
            (TextManager::getInstance() ->isUTF8()
                ? "\xEF\xBB\xBF"    // v UTF8 prida BOM
                : ''
            ) .
            $poms .
            "\n";
    }

    /** Vypise radek s daty do retezce pro CSV soubor.
     * @param $values
     * @return string
     */
    public function outputLine($values)
    {
        $poms = '';

        foreach ($this->fields as $field) {
            $value = null;

            if ($field->sqlFieldIndex >= 0 && isset($values[$field->sqlFieldIndex])) {
                $value = $values[$field->sqlFieldIndex];
            }

            if ($poms) {
                $poms .= ';';
            }

            $poms .= ($field->format ? DBUtils::getFormatedValue($this->database, $field->format, $value) : $value);
        }

        return $poms;
    }

    /** Generuje radky z SQL dotazu do retezce pro CSV soubor.
     * @param Cursor $cursor
     * @return string
     * @throws Exception
     */
    public function outputCursor($cursor)
    {
        $data = '';

        if (!$cursor) {
            throw new Exception('Chyba v metode browser::outputCursor - nejsou nactena data!');
        }

        while ($row = $cursor->fetch()) {
            if ($this->enableLine($row)) {
                $data .= $this->outputLine($row) . "\n";
            }
        }

        return $data;
    }

    /** Posle na vystup obsah browseru ve forme CSV souboru.
     * @throws Exception
     */
    public function output()
    {
        $data =
            $this->outputHeader() .
            $this->outputCursor($this->cursor);

        $csv = new DataOutputObject($data, 'dovednosti.csv', TextManager::getInstance()->getCharSet());

        $csv->output();
    }

}
